/*!
* \file PathStructures.cpp
* \brief Used functions in the library
* \author CSTB
* \version 1.0
*/

#include "PathStructures.h"


/*!
* \brief Computes the distance between 2 2D points
* 
* \param position1 The first point coordinates
* 
* \param position2 The second point coordinates
* 
* \returns the distance between the 2 positions
*/
double distance2D(Position2D const* position1, Position2D const* position2)
{
	return sqrt(pow(position2->d - position1->d, 2) + pow(position2->z - position1->z, 2));
}

/*!
* \brief Computes the distance between 2 3D points
* 
* \param position1 The first point coordinates
* 
* \param position2 The second point coordinates
* 
* \returns the distance between the 2 positions
*/
double distance3D(Position3D const* position1, Position3D const* position2)
{
	return sqrt(pow(position2->x - position1->x, 2) + pow(position2->y - position1->y, 2) + pow(position2->z - position1->z, 2));
}

/*!
* \brief Computes the ground distance between 2 positions (ie distance with x and y, but not z)
* 
* \param position1 The first point coordinates
* 
* \param position2 The second point coordinates
* 
* \returns the ground distance
*/
double GroundDistance(Position3D const* position1, Position3D const* position2)
{
	return sqrt(pow(position2->x - position1->x, 2) + pow(position2->y - position1->y, 2));
}

/*!
* \brief Fills the TerrainItem Position2D
* 
* \param terrain The terrain item
*
* \param source The source
*
* \param cumDistance The cumulated ground distance
*/
void FillPlanePosition(ProfilePointNMPB* terrain, ProfilePointNMPB const* source, double cumDistance)
{
	// Fill the position2D
	terrain->position2D.d = GroundDistance(&source->position3D, &terrain->position3D) + cumDistance;
	terrain->position2D.z = terrain->position3D.z;
}
